<?php

namespace Drupal\ai_provider_drupal_coder\Plugin\AiProvider;

use Drupal\ai\Attribute\AiProvider;
use Drupal\ai\Base\AiProviderClientBase;
use Drupal\ai\Exception\AiResponseErrorException;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatInterface;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\Chat\ChatOutput;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\Yaml\Yaml;

/**
 * Defines the Drupal Coder Provider plugin.
 *
 * This class implements the AI provider plugin for Drupal Coder,
 * connecting to the martelus.store API for Drupal code generation.
 */
#[AiProvider(
  id: 'drupal_coder',
  label: new TranslatableMarkup('Drupal Coder')
)]
class DrupalCoderProvider extends AiProviderClientBase implements ChatInterface {

  /**
   * The API key for authenticating with the Drupal Coder service.
   */
  protected string $apiKey = '';

  /**
   * The base URL for the Drupal Coder API.
   */
  protected string $apiUrl = 'https://martelus.store/api';

  /**
   * Flag to enable or disable moderation of responses.
   */
  protected bool $moderation = TRUE;

  /**
   * Retrieves the list of configured models supported by this provider.
   *
   * @param string|null $operation_type
   *   The operation type, e.g., "chat".
   * @param array $capabilities
   *   Specific capabilities to filter models by.
   *
   * @return array
   *   An array of supported model configurations.
   *
   * @throws AiResponseErrorException
   *   Thrown if the models cannot be fetched.
   */
  public function getConfiguredModels(string $operation_type = NULL, array $capabilities = []): array {
    // Load models from API definition.
    $definition = $this->getApiDefinition();
    
    if ($operation_type && isset($definition[$operation_type]['models'])) {
      return $definition[$operation_type]['models'];
    }
    
    // Combine all models from different operation types.
    $models = [];
    foreach ($definition as $type => $type_config) {
      if (isset($type_config['models'])) {
        $models = array_merge($models, $type_config['models']);
      }
    }
    
    return $models;
  }

  /**
   * Checks if the provider is usable for a given operation type.
   *
   * @param string|null $operation_type
   *   The type of operation, e.g., "chat".
   * @param array $capabilities
   *   Additional capabilities to check against.
   *
   * @return bool
   *   TRUE if the provider can be used; FALSE otherwise.
   */
  public function isUsable(string $operation_type = NULL, array $capabilities = []): bool {
    if (!$this->getConfig()->get('api_key')) {
      return FALSE;
    }

    if ($operation_type) {
      return in_array($operation_type, $this->getSupportedOperationTypes());
    }

    return TRUE;
  }

  /**
   * Returns the operation types supported by this provider.
   *
   * @return array
   *   An array of supported operation types, e.g., ['chat'].
   */
  public function getSupportedOperationTypes(): array {
    return ['chat'];
  }

  /**
   * Retrieves the configuration for this plugin.
   *
   * @return ImmutableConfig
   *   The configuration object.
   */
  public function getConfig(): ImmutableConfig {
    return $this->configFactory->get('ai_provider_drupal_coder.settings');
  }

  /**
   * Retrieves the API definition for this provider.
   *
   * @return array
   *   An array of API defaults defined in the YAML file.
   */
  public function getApiDefinition(): array {
    $definition = Yaml::parseFile(
      $this->moduleHandler->getModule('ai_provider_drupal_coder')
        ->getPath() . '/definitions/api_defaults.yml'
    );
    return $definition;
  }

  /**
   * Configures settings for a specific model.
   *
   * @param string $model_id
   *   The ID of the model being configured.
   * @param array $generalConfig
   *   General configuration options.
   *
   * @return array
   *   The final model settings.
   */
  public function getModelSettings(string $model_id, array $generalConfig = []): array {
    // First try to get model-specific settings from the API definition.
    $definition = $this->getApiDefinition();
    foreach ($definition as $operation_type => $type_config) {
      if (isset($type_config['models'][$model_id]['settings'])) {
        return array_merge(
          $type_config['models'][$model_id]['settings'],
          $generalConfig
        );
      }
    }
    
    return $generalConfig;
  }

  /**
   * Sets the authentication method for the provider.
   *
   * @param mixed $authentication
   *   The API key or other credentials.
   */
  public function setAuthentication(mixed $authentication): void {
    $this->apiKey = $authentication;
  }

  /**
   * Executes a chat operation with the AI model.
   *
   * @param array|string|ChatInput $input
   *   The input messages or configuration for the chat.
   * @param string $model_id
   *   The ID of the model to use.
   * @param array $tags
   *   Optional tags for additional metadata.
   *
   * @return ChatOutput
   *   The response from the AI model.
   *
   * @throws AiResponseErrorException
   *   Thrown if there's an error communicating with the API.
   */
  public function chat(array|string|ChatInput $input, string $model_id, array $tags = []): ChatOutput {
    // Load API key if not already set.
    if (empty($this->apiKey)) {
      $this->apiKey = $this->loadApiKey();
    }
    
    // Load API URL.
    $api_url = $this->getConfig()->get('api_url') ?: 'https://martelus.store/api';
    $this->apiUrl = $api_url;

    // Prepare API request parameters.
    $messages = [];
    if ($input instanceof ChatInput) {
      $messages = array_map(function ($message) {
        return [
          'role' => $message->getRole(),
          'content' => $message->getText(),
        ];
      }, $input->getMessages());
    }
    elseif (is_string($input)) {
      $messages = [
        [
          'role' => 'user',
          'content' => $input,
        ],
      ];
    }
    elseif (is_array($input)) {
      $messages = $input;
    }

    // Get model settings.
    $model_settings = $this->getModelSettings($model_id);

    // Prepare request data.
    $request_data = [
      'model' => $model_id,
      'messages' => $messages,
      'temperature' => $model_settings['temperature'] ?? 0.7,
      'max_tokens' => $model_settings['max_tokens'] ?? 2048,
    ];
    
    // Optional parameters.
    if (isset($model_settings['top_p'])) {
      $request_data['top_p'] = $model_settings['top_p'];
    }
    if (isset($model_settings['frequency_penalty'])) {
      $request_data['frequency_penalty'] = $model_settings['frequency_penalty'];
    }
    if (isset($model_settings['presence_penalty'])) {
      $request_data['presence_penalty'] = $model_settings['presence_penalty'];
    }
    
    try {
      // Make the API request.
      $response = $this->httpClient->post($this->apiUrl . '/chat', [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $this->apiKey,
        ],
        'json' => $request_data,
        'timeout' => 60,
      ]);
      
      // Parse the response.
      $response_body = json_decode((string) $response->getBody(), TRUE);
      
      if (!isset($response_body['content'])) {
        throw new AiResponseErrorException('Invalid response from Drupal Coder API.');
      }
      
      // Create and return a ChatOutput object.
      $message = new ChatMessage('assistant', $response_body['content']);
      return new ChatOutput($message, $response_body, []);
      
    }
    catch (RequestException $e) {
      // Handle API request errors.
      $error_message = $e->getMessage();
      $response_body = $e->getResponse() ? json_decode((string) $e->getResponse()->getBody(), TRUE) : NULL;
      
      if ($response_body && isset($response_body['error'])) {
        $error_message = $response_body['error'];
      }
      
      throw new AiResponseErrorException('Drupal Coder API Error: ' . $error_message);
    }
  }

  /**
   * Enables moderation for all future responses.
   */
  public function enableModeration(): void {
    $this->moderation = TRUE;
  }

  /**
   * Disables moderation for all future responses.
   */
  public function disableModeration(): void {
    $this->moderation = FALSE;
  }

  /**
   * Retrieves the API key from the key module.
   *
   * @return string
   *   The API key value.
   */
  protected function loadApiKey(): string {
    return $this->keyRepository->getKey($this->getConfig()->get('api_key'))
      ->getKeyValue();
  }

  /**
   * Sets the configuration for this provider.
   *
   * @param array $configuration
   *   An array of configuration values.
   */
  public function setConfiguration(array $configuration): void {
    parent::setConfiguration($configuration);
  }

}